/*
 * Copyright 2022 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wDirichlet.h"
#include "wProblem.h"
#include "wTag.h"
#include "wElement.h"
#include "wNode.h"
#include <algorithm>
using namespace heat;

Dirichlet::Dirichlet(std::shared_ptr<MshData> _msh, int no, double _val) : Group(_msh, no), val(_val)
{
    for (auto e : tag->elems)
        for (auto n : e->nodes)
            nodes.push_back(n);
    std::sort(nodes.begin(), nodes.end());
    auto it = std::unique(nodes.begin(), nodes.end());
    nodes.resize(std::distance(nodes.begin(), it));

    // pbl.dBCs.push_back(this);
}

Dirichlet::Dirichlet(std::shared_ptr<MshData> _msh, std::string const &name, double _val) : Group(_msh, name), val(_val)
{
    for (auto e : tag->elems)
        for (auto n : e->nodes)
            nodes.push_back(n);
    std::sort(nodes.begin(), nodes.end());
    auto it = std::unique(nodes.begin(), nodes.end());
    nodes.resize(std::distance(nodes.begin(), it));

    // pbl.dBCs.push_back(this);
}

/**
 *  @brief default implementation (prescribes "val" to all nodes)
 */

void Dirichlet::apply(std::vector<double> &rhs)
{
    for (auto n : nodes)
        rhs[n->row] = val;
}

void Dirichlet::write(std::ostream &out) const
{
    out << "heat::Dirichlet on " << *tag << " (val=" << val << ")\n";
    for (auto n : nodes)
    {
        out << '\t' << *n << '\n';
    }
}
