/*
 * Copyright 2022 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WPERIODIC_H
#define WPERIODIC_H

#include "heat.h"
#include "wObject.h"
#include <vector>
#include <string>
#include <list>
#include <memory>
#include <Eigen/Dense>

using namespace tbox;

namespace heat
{

#ifndef SWIG
class HEAT_API NodePair
{
public:
    Node *n;
    Node *d;
    NodePair(Node *_n, Node *_d, double tol = 1e-12);
    friend HEAT_API std::ostream &operator<<(std::ostream &out, NodePair const &obj);
};
#endif // SWIG

/**
 * @brief manages periodic BCs
 */

class HEAT_API Periodic : public fwk::wSharedObject
{
    std::shared_ptr<heat::Problem> pbl;

public:
#ifndef SWIG
    double TM;
    double dTxM;
    double dTyM;
    std::vector<NodePair> LR;    ///< left-right boundaries
    std::vector<NodePair> BT;    ///< bottom-top boundaries
    std::vector<Node *> corners; ///< corner nodes
#endif
    Eigen::Vector3d qM; ///< calculated mean flux
    bool verbose;       ///< print on/off

    Periodic(std::shared_ptr<heat::Problem> _pbl, double _TM, double _dTxM, double _dTyM);
    virtual ~Periodic() { std::cout << "~Periodic()\n"; }

    void reset(double _TM, double _dTxM, double _dTyM);

#ifndef SWIG
    virtual void write(std::ostream &out) const override;

    void init();
#endif

private:
    std::vector<Node *>
    nodesOnEdge(std::list<Node *> const &nlist, double lc, Eigen::Vector3d const &pt, int c, std::string const &name) const;
};

} // namespace heat

#endif // WPERIODIC_H
