/*
 * Copyright 2022 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "waves.h"
#include "wMshData.h"
#include "wProblem.h"
#include "wForwardEuler.h"
#include "wGmshImport.h"
#include "wGmshExport.h"

// Note that the file strates.msh must be generated from waves/tests/attic/strates.geo using gmsh, and placed in the current directory
int main()
{
    auto msh = std::make_shared<tbox::MshData>();
    tbox::GmshImport *gmshLoader = new tbox::GmshImport(msh);
    tbox::GmshExport *gmshWriter = new tbox::GmshExport(msh);
    gmshLoader->load(std::string("strates.msh"), -1);

    auto pbl = std::make_shared<waves::Problem>(msh);

    waves::ForwardEuler scheme(pbl);
    scheme.start(gmshWriter);

    delete gmshLoader;
    delete gmshWriter;

    return 0;
}
